module digmux(val,clk,reset,AN,CA);
input [15:0] val;
input clk;
input reset;
output [3:0] AN;
output [7:0] CA;

reg [3:0] anr = 4'b0000;
reg [7:0] car = 8'b1111_1111;

reg [15:0] QW = 16'd0;
reg [3:0] addr = 4'b0000;
reg [1:0] dig = 2'b11;

reg [5:0] S = 6'b00_0001;
parameter [5:0] 	init		= 6'b00_0001,
				dload 	= 6'b00_0010,
				cathode 	= 6'b00_0100,
				anode 	= 6'b00_1000,
				dwait 	= 6'b01_0000,
				digit 	= 6'b10_0000;

wire [7:0] dout;
wire uclk = ~clk;

wire SDOUT;
reg [15:0] D;
wire [15:0] Q;
reg CE;
reg PL;

assign SDIN = SDOUT;

assign AN = anr;
assign CA = car;

always @( posedge clk ) begin
	if( reset ) begin
		{ anr, CE, PL } <= 0;
		D <= val;
		dig <= 2'b11;
		car <= 8'b1111_1111;
		S <= init;
	end
	else
	case(S)
		init: begin
			D <= val;
			PL <= 1;
			CE <= 1;
			dig <= 2'b11;
			anr <= 4'b0000;

			S <= dload;
		end
		dload: begin
			PL <= 0;
			CE <= 0;
			addr <= Q[3:0];

			S <= cathode;
		end
		cathode: begin
			car <= dout;

			S <= anode;
		end
		anode: begin
			anr <= { dig == 3, dig == 2, dig == 1, dig == 0 };
		   	QW <= 0;
			CE <= 1;

			S <= dwait;
		end
		dwait: begin
			if( QW == 16'hC350 ) S <= digit;	//1 mS wait @ 50 MHz
//			if( QW == 16'h00FF ) S <= digit;	//for simulation
			else QW <= QW + 1;
			//slip in a shift while we're waiting
			if( QW == 16'h003 ) CE <= 0;
		end
		digit: begin
			anr <= 4'b0000;
			dig <= dig + 1;

			if( ( dig == 2'h2 ) && ( val != D ) )
				S <= init;
			else
				S <= dload;
		end
		default: S <= init;
	endcase
end

b2rom b2rom ( 	 .addr(addr),
			 .clk(uclk), 
			 .dout(dout) );

shifty shifty (
    .CLK(uclk),
    .SDIN(SDIN),
    .SDOUT(SDOUT),
    .P_LOAD(PL),
    .D(D),
    .Q(Q),
    .CE(CE) );

endmodule
