`timescale 1ns / 1ps
////////////////////////////////////////////////////////////////////////////////
// Company: 		 SCIPP
// Engineer:		 Kunal Arya
//
// Create Date:    11:42:07 05/10/05
// Design Name:    fancy_counter
// Module Name:    top
// Project Name:   Fancy Counter
// Target Device:  Spartan: xc2s200e-6, also called xc2v200
// Tool versions:  ISE 7.1
// Description:	 Top-level module for fancy counter for the Digilent board
//                 Designed for the DIO1 expansion
// Dependencies:
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
////////////////////////////////////////////////////////////////////////////////
module top(
	clock, 			 // Input, System clock
	reset, 			 // Input, Counter Reset
	direction, 	    // Input, 0 = count up, 1 = count down
	pause,			 // Input, 0 = go, 1 = pause
	leds_value,     // Output, Counter value
	digits_anode,	 // Output, 7-segment display anode
   digits_cathode, // Output, 7-segment display cathode
);

    input clock;
    input reset;
    input direction;
    input pause;

    output [0:7] leds_value;
	 
	 output [3:0] digits_anode;
	 output [7:0] digits_cathode;

	 // For the binary to bcd module
	 wire bcd_done;
	 wire [15:0] bcd_output;
	 wire bcd_enable = 1'b1;
	 wire bcd_start = 1'b0;

    wire [7:0] counter_output;
	 	 	 
	 // Divide the clock
    reg [24:0] clkdiv_count;

	 // choosing 24 here yields a 2.98 Hz clock
	 wire divided_clock = clkdiv_count[20];

	 // Divide the clock
	 always @(posedge clock ) begin

		 clkdiv_count = clkdiv_count + 1;

	 end

    // Create and instantiate our fancy clock
	 fancy_counter counter0 (
	 	.clock( divided_clock ),
		.reset( reset ),
		.pause( pause ),
		.direction( direction ),
		.counter_value( counter_output )
    );

    // Attach the net leds_value to our counter's output
	 assign leds_value = counter_output;

    // Code courtesy Jason Heimann, slightly modified for use here
	 binary_to_bcd btb (
	 	.clk_i( clock ),						// Main clock
		.ce_i( bcd_enable) ,					// Always enabled
		.rst_i( reset ),                 // Attach this to the system-wide reset
		.start_i( divided_clock ),	      // We want to update this periodically, but not at 50 MHz
		.dat_binary_i( counter_output ), // Attach this to the output of the counter
		.dat_bcd_o( bcd_output ),        // Attach the output to the input (below) to the digit_converter
		.done_o( bcd_done )					// We don't need to worry about when this is done
	 );
	
	 // This will take the BCD value and convert into 7-segment display wires
	 digmux digit_converter (
	 	.val( bcd_output ),      // Take the value of the binary->bcd converter
		.clk( clock ),           // Main clock
		.reset( reset ),			 // System reset
		.AN( digits_anode ),     // These signals control which digits are on
		.CA( digits_cathode )    // These signals control which lines on each digit are on
    );

endmodule
